--System Configuration DB Update patch v2.0.5
-- Changes w.r.t. v 2.0.4
-- 1.- Drop Unique constraints from tables with colunms valid_from and valid_until
-- 2.- Create a new table to keep track of the mapping of Windows to Linux paths.
-- 3.- Drop constraint on uniqueness of system name and system number.
-- 4.- Modification of View fw_sys_stat_proj_comps in order to avoid duplication of components

--Update schema version
CREATE OR REPLACE VIEW fw_sys_stat_schema (version) AS
	SELECT '2.0.5'
	FROM dual;

BEGIN
	FOR CONSTRAINTS_RECORDS IN
		(SELECT CONSTRAINT_NAME
		 FROM USER_CONS_COLUMNS
		 WHERE TABLE_NAME = 'FW_SYS_STAT_COMPUTER'
		 AND CONSTRAINT_NAME LIKE 'SYS_C%') LOOP

		 EXECUTE IMMEDIATE 'ALTER TABLE FW_SYS_STAT_COMPUTER DROP CONSTRAINT ' || CONSTRAINTS_RECORDS.CONSTRAINT_NAME;

	END LOOP;
END;
/

--Put back NOT NULL Constraint for hostname on the fw_sys_stat_computer table:
ALTER TABLE FW_SYS_STAT_COMPUTER ADD CONSTRAINT HOSTNAME_NOT_NULL CHECK (HOSTNAME IS NOT NULL);

--Drop constraint on uniqueness of system name, system number, redundancy number.
ALTER TABLE FW_SYS_STAT_PVSS_SYSTEM DROP CONSTRAINT NAME_NUMBER_REDUNDANCY_UQ;

--Create Windows - Linux path mapping table e.g. P:\ in Windows = /afs/cern.ch/ under Linux

CREATE TABLE FW_SYS_STAT_PATH_MAPPING
	(ID				NUMBER,
	 WINDOWS_PATH 	VARCHAR2(512),
	 LINUX_PATH 	VARCHAR2(512),
	 CONSTRAINT PATH_MAPPING_ID_PK PRIMARY KEY (ID),
	 CONSTRAINT PATH_MAPPING_UQ UNIQUE(WINDOWS_PATH, LINUX_PATH));


CREATE SEQUENCE fw_sys_stat_path_mapping_sq
    MINVALUE 1
    MAXVALUE 9999999
    START WITH 1
    INCREMENT BY 1
    NOCACHE;

--Add column to table FW_SYS_STAT_PROJ_INSTALL_COMPS to feedback through the system configuration DB whether the installation was successfull
ALTER TABLE FW_SYS_STAT_PROJ_INSTALL_COMPS
ADD (INSTALLATION_OK NUMBER);
                                     

CREATE OR REPLACE VIEW fw_sys_stat_proj_comps AS
	SELECT HOSTNAME, PROJECT_NAME, COMPONENT_NAME, MAX(COMPONENT_VERSION) COMPONENT_VERSION, IS_SUBCOMPONENT, DEFAULT_PATH, IS_OFFICIAL, IS_PATCH, DESCRIPTION_FILE, OVERWRITE_FILES, FORCE_REQUIRED, IS_SILENT, MAX(VALID_FROM) as M_VALID_FROM
	FROM (SELECT PC.HOSTNAME, P.PROJECT_NAME, C.COMPONENT_NAME, C.COMPONENT_VERSION, C.IS_SUBCOMPONENT, C.DEFAULT_PATH, C.IS_OFFICIAL, C.IS_PATCH, GC.DESCRIPTION_FILE, PG.OVERWRITE_FILES, PG.FORCE_REQUIRED, PG.IS_SILENT, GC.VALID_FROM
			FROM FW_SYS_STAT_PVSS_PROJECT P, FW_SYS_STAT_GROUP_OF_COMP G, FW_SYS_STAT_PROJECT_GROUPS PG, FW_SYS_STAT_COMP_IN_GROUPS GC, FW_SYS_STAT_COMPONENT C, FW_SYS_STAT_COMPUTER PC
			WHERE P.ID = PG.PROJECT_ID AND PG.VALID_UNTIL IS NULL AND P.VALID_UNTIL IS NULL AND G.ID = PG.GROUP_ID AND GC.GROUP_ID = G.ID AND GC.VALID_UNTIL IS NULL AND C.ID = GC.FW_COMPONENT_ID AND C.VALID_UNTIL IS NULL AND PC.ID = P.COMPUTER_ID AND PC.VALID_UNTIL IS NULL)
	GROUP BY COMPONENT_NAME, HOSTNAME, PROJECT_NAME, COMPONENT_NAME, IS_SUBCOMPONENT, DEFAULT_PATH, IS_OFFICIAL, IS_PATCH, DESCRIPTION_FILE, OVERWRITE_FILES, FORCE_REQUIRED, IS_SILENT
	ORDER BY M_VALID_FROM;