---- fw_sys_stat_sas_target_db_pkg BODY ----

CREATE OR REPLACE PACKAGE BODY FW_SYS_STAT_SAS_TARGET_DB_PKG AS

PROCEDURE CREATE_PROJECT(PROJ_NAME IN VARCHAR2,
                         SYSTEM_NAME IN VARCHAR2,
                         SYSTEM_NUMBER  IN NUMBER,
                         PMON_PORT IN NUMBER,
                         DATA_PORT IN NUMBER,
                         EVENT_PORT IN NUMBER,
                         DIST_PORT IN NUMBER,
                         REDU_PORT IN NUMBER,
                         SPLIT_PORT IN NUMBER,
                         HOSTNAME IN VARCHAR2,
                         REDU_HOSTNAME IN VARCHAR2,
                         IS_REDUNDANT IN BOOLEAN,
                         REDU_PEER IN NUMBER,
                         RESPONSIBLE IN VARCHAR2,
                         PVSS_VERSION IN VARCHAR2,
                         CREATED_BY IN VARCHAR2,
                         SERVICE_ACCOUNT IN VARCHAR2,
                         JIRA_ISSUE IN VARCHAR2) AS
    system_id  number;
    computer_id number := 1;
    redu_computer_id number := 1;
    project_id number;
    error_message varchar2(1000);
BEGIN
    COMMIT;
    SET TRANSACTION NAME 'new_project';
    INSERT INTO fw_sys_stat_pvss_system(id, system_name, system_number, data_port, event_port, dist_port, valid_from, redu_port, split_port)
    VALUES (FW_SYS_STAT_PVSS_SYSTEM_SQ.NEXTVAL, SYSTEM_NAME, SYSTEM_NUMBER, DATA_PORT, EVENT_PORT, DIST_PORT, CURRENT_DATE, REDU_PORT, SPLIT_PORT)
    RETURNING id INTO system_id;

    computer_id := GET_OR_CREATE_COMPUTER(HOSTNAME);
    IF IS_REDUNDANT THEN
        redu_computer_id := GET_OR_CREATE_COMPUTER(REDU_HOSTNAME);
    ELSE
        redu_computer_id := computer_id;
    END IF;

    INSERT INTO fw_sys_stat_pvss_project (id, computer_id, project_name, pmon_port, system_id, redu_computer_id, responsible, pvss_version, service_account, created_by, status)
    VALUES (FW_SYS_STAT_PVSS_PROJECT_SQ.NEXTVAL, computer_id, PROJ_NAME, PMON_PORT, system_id, redu_computer_id, RESPONSIBLE, PVSS_VERSION, SERVICE_ACCOUNT, CREATED_BY, 'CREATION_PENDING')
    RETURNING id INTO project_id;

    INSERT INTO fw_sys_stat_project_status(id, project_id, computer_id, last_time_checked, need_synchronize)
    VALUES (fw_sys_stat_project_status_sq.NEXTVAL, project_id, computer_id, sysdate, 'Y');

    IF IS_REDUNDANT THEN
          INSERT INTO fw_sys_stat_project_status(id, project_id, computer_id, need_synchronize)
          VALUES (fw_sys_stat_project_status_sq.NEXTVAL, project_id, redu_computer_id, 'Y');
    END IF;

    COMMIT;
EXCEPTION
WHEN OTHERS THEN
    error_message := 'some error, nothing should be written in db ' || SQLCODE || ' -ERROR- ' || SQLERRM;
    ROLLBACK;
    RAISE;
END CREATE_PROJECT;

FUNCTION GET_OR_CREATE_COMPUTER(COMPUTER_HOSTNAME IN VARCHAR2) RETURN NUMBER AS
    computer_count number := 0;
    computer_id number := 0;
BEGIN
    SELECT count(id) INTO computer_count FROM fw_sys_stat_computer computer WHERE upper(computer.hostname) = upper(COMPUTER_HOSTNAME) AND valid_until IS NULL;
    IF computer_count = 0 THEN
        INSERT INTO fw_sys_stat_computer (id, hostname)
        VALUES (fw_sys_stat_computer_sq.nextval, upper(COMPUTER_HOSTNAME))
        RETURNING id INTO computer_id;
    ELSE
        SELECT id INTO computer_id FROM fw_sys_stat_computer computer WHERE upper(computer.hostname) = upper(COMPUTER_HOSTNAME) AND valid_until IS NULL;
    END IF;
    RETURN computer_id;
END GET_OR_CREATE_COMPUTER;

PROCEDURE UPDATE_PROJECT_INFO(PROJECT_ID IN NUMERIC,
                              RESPONSIBLE IN VARCHAR2,
                              STATUS IN VARCHAR2,
                              COMMENT_TEXT IN VARCHAR2,
                              JIRA_ISSUE IN VARCHAR2,
                              WCCOA_URL IN VARCHAR2,
                              INFO_URL IN VARCHAR2,
                              APP_DOMAIN IN VARCHAR2,
                              SERVICE_ACCOUNT IN VARCHAR2) AS
    l_domain_id number := null;
BEGIN
    if length(APP_DOMAIN) > 0 then
        begin
            select id into l_domain_id from FW_SYS_STAT_APPLICATION_DOMAIN where name = UPDATE_PROJECT_INFO.APP_DOMAIN;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            raise_application_error(-20000, 'Selected application domain "' || APP_DOMAIN || '" cannot be found');
        end;
    end if;
    
    UPDATE
        fw_sys_stat_pvss_project proj
    SET 
        proj.responsible = UPDATE_PROJECT_INFO.RESPONSIBLE,
        proj.status = UPDATE_PROJECT_INFO.STATUS,
        proj.comment_text = UPDATE_PROJECT_INFO.COMMENT_TEXT,
        proj.creation_jira_issue = UPDATE_PROJECT_INFO.JIRA_ISSUE,
        proj.wccoa_url = UPDATE_PROJECT_INFO.WCCOA_URL,
        proj.info_url = UPDATE_PROJECT_INFO.INFO_URL,
        proj.application_domain_id = l_domain_id,
        proj.service_account = UPDATE_PROJECT_INFO.SERVICE_ACCOUNT
    WHERE
        proj.id = PROJECT_ID;  
END UPDATE_PROJECT_INFO;

PROCEDURE DELETE_PROJECT(PROJECT_ID IN NUMERIC) AS
    -- computer_id number := -1;
    system_id number := -1;
BEGIN
    SELECT system_id into DELETE_PROJECT.system_id FROM FW_SYS_STAT_PVSS_PROJECT WHERE id = PROJECT_ID;
    DELETE FROM FW_SYS_STAT_PVSS_PROJECT WHERE id = PROJECT_ID;
    DELETE_SYSTEM(system_id);
    -- DELETE_COMPUTER(computer_id);
END DELETE_PROJECT;

PROCEDURE DELETE_SYSTEM(SYSTEM_ID IN NUMERIC) AS
    l_system_id NUMERIC := SYSTEM_ID;
BEGIN
    DELETE FROM FW_SYS_STAT_PVSS_SYSTEM WHERE id = l_system_id AND (SELECT count(id) FROM fw_sys_stat_pvss_project WHERE system_id = l_system_id) = 0;
END DELETE_SYSTEM;

PROCEDURE DELETE_COMPUTER(COMPUTER_ID IN NUMERIC) AS
    l_computer_id NUMERIC := COMPUTER_ID;
BEGIN
    DELETE FROM FW_SYS_STAT_COMPUTER WHERE id = l_computer_id AND (SELECT count(id) FROM fw_sys_stat_pvss_project WHERE computer_id = l_computer_id) = 0 AND (SELECT count(id) FROM fw_sys_stat_pvss_project WHERE redu_computer_id = l_computer_id) = 0;
END DELETE_COMPUTER;

PROCEDURE INVALIDATE_PROJECT(PROJECT_ID IN NUMERIC) AS
    system_id number := -1;
BEGIN
    SELECT system_id into INVALIDATE_PROJECT.system_id FROM FW_SYS_STAT_PVSS_PROJECT WHERE id = PROJECT_ID;
    UPDATE fw_sys_stat_pvss_project SET valid_until = SYSDATE WHERE id = PROJECT_ID;
    INVALIDATE_SYSTEM(system_id);
END INVALIDATE_PROJECT;

PROCEDURE INVALIDATE_SYSTEM(SYSTEM_ID IN NUMERIC) AS
    l_system_id number := SYSTEM_ID;
BEGIN
    UPDATE fw_sys_stat_pvss_system SET valid_until = SYSDATE WHERE id = l_system_id AND (SELECT count(id) FROM fw_sys_stat_pvss_project WHERE valid_until is null AND system_id = l_system_id) = 0;
END INVALIDATE_SYSTEM;

FUNCTION IS_DOMAIN_RESPONSIBLE(USERNAME IN VARCHAR2, PROJECT_ID IN NUMERIC) RETURN BOOLEAN AS
    responsible VARCHAR2(100) := '';
BEGIN
    BEGIN
        SELECT domain.contact_person INTO IS_DOMAIN_RESPONSIBLE.responsible FROM fw_sys_stat_pvss_project project, fw_sys_stat_application_domain domain WHERE project.application_domain_id = domain.id and project.id = PROJECT_ID;
    EXCEPTION
    WHEN NO_DATA_FOUND THEN
        IS_DOMAIN_RESPONSIBLE.responsible := '';
    end;
    RETURN upper(responsible)=upper(USERNAME);
END IS_DOMAIN_RESPONSIBLE;

FUNCTION GET_NEW_SYSTEM_NUMBER RETURN NUMBER AS
    system_nr NUMBER := -1;
BEGIN
    SELECT MIN(r) INTO system_nr FROM (SELECT ROWNUM r FROM dual CONNECT BY ROWNUM <= 2048) WHERE r NOT IN (SELECT system_number FROM fw_sys_stat_pvss_system);
    RETURN system_nr;
END GET_NEW_SYSTEM_NUMBER;

END FW_SYS_STAT_SAS_TARGET_DB_PKG;
